@@ -308,8 +308,8 @@ const char dirconf[]=".jugglucorc";
 //bool exportscans(int handle, const std::span<const ScanData>  (SensorGlucoseData::*proc)(void) const) ;
 
 template <bool repeatids>
-bool exportscans(int handle,  CurData  (SensorGlucoseData::*proc)(const uint32_t,const uint32_t) const,uint32_t starttime=0,uint32_t endtime=UINT32_MAX,int maxcount=INT_MAX) ;
-bool exporthistory(int handle,uint32_t starttime=0,uint32_t endtime=UINT32_MAX,int maxcount=INT_MAX) ;
+bool exportscans(int handle,  CurData  (SensorGlucoseData::*proc)(const uint32_t,const uint32_t) const,uint32_t starttime=0,uint32_t endtime=UINT32_MAX,int maxcount=INT_MAX,bool calibrated=false) ;
+bool exporthistory(int handle,uint32_t starttime=0,uint32_t endtime=UINT32_MAX,int maxcount=INT_MAX,bool calibrated=false) ;
 void showversion() {
 #include "version.h"
     cout<<"Version "<< APPVERSION <<endl;
@@ -809,7 +809,7 @@ extern void setDeactivated(int index,bool deactive) ;
             did=true;
             }
         if(historyexport)  {
-             if(!exportdata(historyexport,exporthistory,starttime,endtime,maxcount))
+             if(!exportdata(historyexport,exporthistory,starttime,endtime,maxcount,true))
                 return 13;
             did=true;
 
@@ -820,12 +820,12 @@ extern void setDeactivated(int index,bool deactive) ;
             did=true;
             }
         if(scanexport) {
-            if(!exportdata(scanexport,exportscans<true>, &SensorGlucoseData::scanInperiod,starttime,endtime,maxcount))
+            if(!exportdata(scanexport,exportscans<true>, &SensorGlucoseData::scanInperiod,starttime,endtime,maxcount,true))
                 return 13;
             did=true;
             }
         if(pollexport) {
-            if(!exportdata(pollexport,exportscans<false>, &SensorGlucoseData::streamInperiod,starttime,endtime,maxcount))
+            if(!exportdata(pollexport,exportscans<false>, &SensorGlucoseData::streamInperiod,starttime,endtime,maxcount,true))
                 return 13;
             did=true;
             }
